@@ -102,6 +102,20 @@ void AlkFinanceQuoteProcess::slotProcessExited()
     d->m_isDone = true;
 }
 
+void AlkFinanceQuoteProcess::testLaunch(const QString& scriptPath)
+{
+    clearProgram();
+
+    qDebug() << "running perl" << scriptPath << "-t";
+
+    *this << "perl" << scriptPath << "-t";
+    setOutputChannelMode(KProcess::OnlyStdoutChannel);
+    start();
+    if (!waitForStarted()) {
+        qWarning("Unable to start FQ script");
+    }
+}
+
 void AlkFinanceQuoteProcess::launch(const QString &scriptPath)
 {
     clearProgram();
@@ -123,7 +137,11 @@ bool AlkFinanceQuoteProcess::isFinished() const
 
 const QStringList AlkFinanceQuoteProcess::getSourceList() const
 {
+    #if QT_VERSION < QT_VERSION_CHECK(5,14,0)
     QStringList raw = d->m_string.split(0x0A, QString::SkipEmptyParts);
+    #else
+    QStringList raw = d->m_string.split(0x0A, Qt::SkipEmptyParts);
+    #endif
     QStringList sources;
     QStringList::iterator it;
     for (it = raw.begin(); it != raw.end(); ++it) {
