@@ -40,6 +40,7 @@
 
 #if QT_VERSION >= QT_VERSION_CHECK(5,0,0)
     #include <KLocalizedString>
+    #include <QRegularExpression>
 #ifndef BUILD_WITH_QTNETWORK
     #include <KIO/Job>
 #endif
@@ -64,21 +65,29 @@
 #include <KProcess>
 #include <KShell>
 
+#if QT_VERSION < QT_VERSION_CHECK(5,0,0)
+    using Regex = QRegExp;
+#else
+    using Regex = QRegularExpression;
+#endif
+
+#ifndef I18N_NOOP
+#include <KLazyLocalizedString>
+#endif
+
 AlkOnlineQuote::Errors::Errors()
 {
 }
 
 AlkOnlineQuote::Errors::Errors(Type type)
 {
-    m_type.append(type);
+    if (type != None)
+        m_type.append(type);
+    else
+        m_type.clear();
 }
 
-AlkOnlineQuote::Errors::Errors(const Errors &e)
-{
-    m_type = e.m_type;
-}
-
-AlkOnlineQuote::Errors &AlkOnlineQuote::Errors::operator |=(Type t)
+AlkOnlineQuote::Errors& AlkOnlineQuote::Errors::operator|=(Type t)
 {
     if (!m_type.contains(t)) {
         m_type.append(t);
@@ -101,6 +110,8 @@ public:
     QString m_quoteData;
     QString m_symbol;
     QString m_id;
+    QDate m_startDate;
+    QDate m_endDate;
     QDate m_date;
     double m_price;
     AlkOnlineQuoteSource m_source;
@@ -110,6 +121,7 @@ public:
     QString m_acceptLanguage;
     AlkOnlineQuotesProfile *m_profile;
     bool m_ownProfile;
+    int m_timeout;
 
 #if QT_VERSION < QT_VERSION_CHECK(5,0,0)
     static int dbgArea()
@@ -119,10 +131,11 @@ public:
     }
 #endif
 
-    Private(AlkOnlineQuote *parent)
+    explicit Private(AlkOnlineQuote *parent)
         : m_p(parent)
         , m_eventLoop(nullptr)
         , m_ownProfile(false)
+        , m_timeout(-1)
     {
         connect(&m_filter, SIGNAL(processExited(QString)), this, SLOT(slotParseQuote(QString)));
     }
@@ -138,21 +151,25 @@ public:
                                  const QString &_source);
     bool launchWebKitHtmlParser(const QString &_symbol, const QString &_id, const QString &_source);
     bool launchNative(const QString &_symbol, const QString &_id, const QString &_source);
-    bool launchFinanceQuote(const QString &_symbol, const QString &_id, const QString &_source);
-    void enter_loop();
+    bool launchFinanceQuote(const QString& _symbol, const QString& _id, const QString& _source);
     bool parsePrice(const QString &pricestr);
     bool parseDate(const QString &datestr);
+    bool parseQuoteStripHTML(const QString &quotedata);
+    bool parseQuoteHTML(const QString &quotedata);
+    bool parseQuoteCSV(const QString &quotedata);
     bool downloadUrl(const KUrl& url);
     bool processDownloadedFile(const KUrl& url, const QString& tmpFile);
     bool processDownloadedPage(const KUrl &url, const QByteArray &page);
+    bool processLocalScript(const KUrl& url);
 
-public slots:
+public Q_SLOTS:
     void slotLoadStarted();
     void slotLoadFinishedHtmlParser(bool ok = false);
     void slotLoadFinishedCssSelector(bool ok);
-    bool slotParseQuote(const QString &_quotedata);
+    bool slotParseQuote(const QString &quotedata);
 
-private slots:
+private Q_SLOTS:
+    void slotLoadTimeout();
 #ifndef BUILD_WITH_QTNETWORK
     void downloadUrlDone(KJob* job);
 #else
@@ -166,7 +183,7 @@ bool AlkOnlineQuote::Private::initLaunch(const QString &_symbol, const QString &
     m_id = _id;
     m_errors = Errors::None;
 
-    emit m_p->status(QString("(Debug) symbol=%1 id=%2...").arg(_symbol, _id));
+    Q_EMIT m_p->status(QString("(Debug) symbol=%1 id=%2...").arg(_symbol, _id));
 
     // Get sources from the config file
     QString source = _source;
@@ -175,7 +192,7 @@ bool AlkOnlineQuote::Private::initLaunch(const QString &_symbol, const QString &
     }
 
     if (!m_profile->quoteSources().contains(source)) {
-        emit m_p->error(i18n("Source <placeholder>%1</placeholder> does not exist.", source));
+        Q_EMIT m_p->error(i18n("Source <placeholder>%1</placeholder> does not exist.", source));
         m_errors |= Errors::Source;
         return false;
     }
@@ -194,7 +211,7 @@ bool AlkOnlineQuote::Private::initLaunch(const QString &_symbol, const QString &
         if (splitrx.indexIn(m_symbol) != -1) {
             url = KUrl(m_source.url().arg(splitrx.cap(1), splitrx.cap(2)));
         } else {
-            kDebug(Private::dbgArea()) << "WebPriceQuote::launch() did not find 2 symbols";
+            kDebug(Private::dbgArea()) << QString("AlkOnlineQuote::Private::initLaunch() did not find 2 symbols in '%1'").arg(m_symbol);
         }
     } else {
         // a regular one-symbol quote
@@ -209,9 +226,9 @@ bool AlkOnlineQuote::Private::initLaunch(const QString &_symbol, const QString &
 void AlkOnlineQuote::Private::slotLoadFinishedHtmlParser(bool ok)
 {
     if (!ok) {
-        emit m_p->error(i18n("Unable to fetch url for %1", m_symbol));
+        Q_EMIT m_p->error(i18n("Unable to fetch url for %1", m_symbol));
         m_errors |= Errors::URL;
-        emit m_p->failed(m_id, m_symbol);
+        Q_EMIT m_p->failed(m_id, m_symbol);
     } else {
         // parse symbol
         slotParseQuote(AlkOnlineQuotesProfileManager::instance().webPage()->toHtml());
@@ -223,32 +240,32 @@ void AlkOnlineQuote::Private::slotLoadFinishedHtmlParser(bool ok)
 void AlkOnlineQuote::Private::slotLoadFinishedCssSelector(bool ok)
 {
     if (!ok) {
-        emit m_p->error(i18n("Unable to fetch url for %1", m_symbol));
+        Q_EMIT m_p->error(i18n("Unable to fetch url for %1", m_symbol));
         m_errors |= Errors::URL;
-        emit m_p->failed(m_id, m_symbol);
+        Q_EMIT m_p->failed(m_id, m_symbol);
     } else {
         AlkWebPage *webPage = AlkOnlineQuotesProfileManager::instance().webPage();
         // parse symbol
-        QString symbol = webPage->getFirstElement(m_source.sym());
-        if (!symbol.isEmpty()) {
-            emit m_p->status(i18n("Symbol found: '%1'", symbol));
+        QString identifier = webPage->getFirstElement(m_source.idRegex());
+        if (!identifier.isEmpty()) {
+            Q_EMIT m_p->status(i18n("Symbol found: '%1'", identifier));
         } else {
             m_errors |= Errors::Symbol;
-            emit m_p->error(i18n("Unable to parse symbol for %1", m_symbol));
+            Q_EMIT m_p->error(i18n("Unable to parse symbol for %1", m_symbol));
         }
 
         // parse price
-        QString price = webPage->getFirstElement(m_source.price());
+        QString price = webPage->getFirstElement(m_source.priceRegex());
         bool gotprice = parsePrice(price);
 
         // parse date
-        QString date = webPage->getFirstElement(m_source.date());
+        QString date = webPage->getFirstElement(m_source.dateRegex());
         bool gotdate = parseDate(date);
 
         if (gotprice && gotdate) {
-            emit m_p->quote(m_id, m_symbol, m_date, m_price);
+            Q_EMIT m_p->quote(m_id, m_symbol, m_date, m_price);
         } else {
-            emit m_p->failed(m_id, m_symbol);
+            Q_EMIT m_p->failed(m_id, m_symbol);
         }
     }
     if (m_eventLoop)
@@ -257,7 +274,7 @@ void AlkOnlineQuote::Private::slotLoadFinishedCssSelector(bool ok)
 
 void AlkOnlineQuote::Private::slotLoadStarted()
 {
-    emit m_p->status(i18n("Fetching URL %1...", m_url.prettyUrl()));
+    Q_EMIT m_p->status(i18n("Fetching URL %1...", m_url.prettyUrl()));
 }
 
 bool AlkOnlineQuote::Private::launchWebKitCssSelector(const QString &_symbol, const QString &_id,
@@ -270,7 +287,8 @@ bool AlkOnlineQuote::Private::launchWebKitCssSelector(const QString &_symbol, co
     connect(webPage, SIGNAL(loadStarted()), this, SLOT(slotLoadStarted()));
     connect(webPage, SIGNAL(loadFinished(bool)), this,
             SLOT(slotLoadFinishedCssSelector(bool)));
-    QTimer::singleShot(20000, this, SLOT(slotLoadFinishedHtmlParser()));
+    if (m_timeout != -1)
+        QTimer::singleShot(m_timeout, this, SLOT(slotLoadTimeout()));
     webPage->setUrl(m_url);
     m_eventLoop = new QEventLoop;
     m_eventLoop->exec();
@@ -293,7 +311,8 @@ bool AlkOnlineQuote::Private::launchWebKitHtmlParser(const QString &_symbol, con
     AlkWebPage *webPage = AlkOnlineQuotesProfileManager::instance().webPage();
     connect(webPage, SIGNAL(loadStarted()), this, SLOT(slotLoadStarted()));
     connect(webPage, SIGNAL(loadFinished(bool)), this, SLOT(slotLoadFinishedHtmlParser(bool)));
-    QTimer::singleShot(20000, this, SLOT(slotLoadFinishedHtmlParser()));
+    if (m_timeout != -1)
+        QTimer::singleShot(m_timeout, this, SLOT(slotLoadTimeout()));
     webPage->load(m_url, m_acceptLanguage);
     m_eventLoop = new QEventLoop;
     m_eventLoop->exec();
@@ -316,22 +335,7 @@ bool AlkOnlineQuote::Private::launchNative(const QString &_symbol, const QString
 
     KUrl url = m_url;
     if (url.isLocalFile()) {
-        emit m_p->status(i18nc("The process x is executing", "Executing %1...", url.toLocalFile()));
-
-        m_filter.clearProgram();
-        m_filter << url.toLocalFile().split(' ', QString::SkipEmptyParts);
-        m_filter.setSymbol(m_symbol);
-
-        m_filter.setOutputChannelMode(KProcess::MergedChannels);
-        m_filter.start();
-
-        if (m_filter.waitForStarted()) {
-            result = true;
-        } else {
-            emit m_p->error(i18n("Unable to launch: %1", url.toLocalFile()));
-            m_errors |= Errors::Script;
-            result = slotParseQuote(QString());
-        }
+        result = processLocalScript(url);
     } else {
         slotLoadStarted();
         result = downloadUrl(url);
@@ -339,6 +343,32 @@ bool AlkOnlineQuote::Private::launchNative(const QString &_symbol, const QString
     return result;
 }
 
+bool AlkOnlineQuote::Private::processLocalScript(const KUrl& url)
+{
+    Q_EMIT m_p->status(i18nc("The process x is executing", "Executing %1...", url.toLocalFile()));
+
+    bool result = true;
+
+    m_filter.clearProgram();
+#if QT_VERSION < QT_VERSION_CHECK(5, 14, 0)
+    m_filter << url.toLocalFile().split(' ', QString::SkipEmptyParts);
+#else
+    m_filter << url.toLocalFile().split(' ', Qt::SkipEmptyParts);
+#endif
+    m_filter.setSymbol(m_symbol);
+
+    m_filter.setOutputChannelMode(KProcess::MergedChannels);
+    m_filter.start();
+
+    // This seems to work best if we just block until done.
+    if (!m_filter.waitForFinished()) {
+        Q_EMIT m_p->error(i18n("Unable to launch: %1", url.toLocalFile()));
+        m_errors |= Errors::Script;
+        result = slotParseQuote(QString());
+    }
+    return result;
+}
+
 bool AlkOnlineQuote::Private::processDownloadedFile(const KUrl& url, const QString& tmpFile)
 {
   bool result = false;
@@ -350,7 +380,7 @@ bool AlkOnlineQuote::Private::processDownloadedFile(const KUrl& url, const QStri
     result = processDownloadedPage(url, page);
     f.close();
   } else {
-    emit m_p->error(i18n("Failed to open downloaded file"));
+    Q_EMIT m_p->error(i18n("Failed to open downloaded file"));
     m_errors |= Errors::URL;
     result = slotParseQuote(QString());
   }
@@ -367,7 +397,7 @@ bool AlkOnlineQuote::Private::processDownloadedPage(const KUrl& url, const QByte
       codec = QTextCodec::codecForLocale();
     }
     QString quote = codec->toUnicode(page);
-    emit m_p->status(i18n("URL found: %1...", url.prettyUrl()));
+    Q_EMIT m_p->status(i18n("URL found: %1...", url.prettyUrl()));
     if (AlkOnlineQuotesProfileManager::instance().webPageEnabled())
       AlkOnlineQuotesProfileManager::instance().webPage()->setContent(quote.toLocal8Bit());
     result = slotParseQuote(quote);
@@ -391,6 +421,8 @@ bool AlkOnlineQuote::Private::downloadUrl(const QUrl& url)
     KJob *job = KIO::file_copy(url, tmpFileName, -1, KIO::HideProgressInfo);
     connect(job, SIGNAL(result(KJob*)), this, SLOT(downloadUrlDone(KJob*)));
 
+    if (m_timeout != -1)
+        QTimer::singleShot(m_timeout, this, SLOT(slotLoadTimeout()));
     auto result = m_eventLoop->exec(QEventLoop::ExcludeUserInputEvents);
     delete m_eventLoop;
     m_eventLoop = nullptr;
@@ -408,7 +440,7 @@ void AlkOnlineQuote::Private::downloadUrlDone(KJob* job)
     qDebug() << "Downloaded" << tmpFileName << "from" << url;
     result = processDownloadedFile(url, tmpFileName);
   } else {
-    emit m_p->error(job->errorString());
+    Q_EMIT m_p->error(job->errorString());
     m_errors |= Errors::URL;
     result = slotParseQuote(QString());
   }
@@ -436,7 +468,7 @@ bool AlkOnlineQuote::Private::downloadUrl(const KUrl& url)
         result = processDownloadedFile(url, tmpFile);
         KIO::NetAccess::removeTempFile(tmpFile);
     } else {
-        emit m_p->error(KIO::NetAccess::lastErrorString());
+        Q_EMIT m_p->error(KIO::NetAccess::lastErrorString());
         m_errors |= Errors::URL;
         result = slotParseQuote(QString());
     }
@@ -454,15 +486,19 @@ void AlkOnlineQuote::Private::downloadUrlDone(QNetworkReply *reply)
         if (!newUrl.isEmpty() && newUrl != reply->url()) {
             m_url = reply->url().resolved(newUrl);
             // TODO migrate to i18n()
-            emit m_p->status(QString("<font color=\"orange\">%1</font>")
+            Q_EMIT m_p->status(QString("<font color=\"orange\">%1</font>")
+            #ifdef I18N_NOOP
                             .arg(I18N_NOOP("The URL has been redirected; check an update of the online quote URL")));
+            #else
+                            .arg(kli18n("The URL has been redirected; check an update of the online quote URL").untranslatedText()));
+            #endif
             result = 2;
         } else {
             kDebug(Private::dbgArea()) << "Downloaded data from" << reply->url();
             result = processDownloadedPage(KUrl(reply->url()), reply->readAll()) ? 0 : 1;
         }
     } else {
-        emit m_p->error(reply->errorString());
+        Q_EMIT m_p->error(reply->errorString());
         m_errors |= Errors::URL;
         result = slotParseQuote(QString()) ? 0 : 1;
     }
@@ -478,15 +514,21 @@ bool AlkOnlineQuote::Private::downloadUrl(const KUrl &url)
     request.setUrl(url);
     request.setRawHeader("User-Agent", "alkimia " ALK_VERSION_STRING);
     manager.get(request);
+
+    if (m_timeout != -1)
+        QTimer::singleShot(m_timeout, this, SLOT(slotLoadTimeout()));
     m_eventLoop = new QEventLoop;
     int result = m_eventLoop->exec(QEventLoop::ExcludeUserInputEvents);
     delete m_eventLoop;
     m_eventLoop = nullptr;
     if (result == 2) {
-        QNetworkRequest request;
-        request.setUrl(m_url);
-        request.setRawHeader("User-Agent", "alkimia " ALK_VERSION_STRING);
-        manager.get(request);
+        QNetworkRequest req;
+        req.setUrl(m_url);
+        req.setRawHeader("User-Agent", "alkimia " ALK_VERSION_STRING);
+        manager.get(req);
+
+        if (m_timeout != -1)
+            QTimer::singleShot(m_timeout, this, SLOT(slotLoadTimeout()));
         m_eventLoop = new QEventLoop;
         result = m_eventLoop->exec(QEventLoop::ExcludeUserInputEvents);
         delete m_eventLoop;
@@ -504,13 +546,15 @@ bool AlkOnlineQuote::Private::launchFinanceQuote(const QString &_symbol, const Q
     m_symbol = _symbol;
     m_id = _id;
     m_errors = Errors::None;
-    m_source = AlkOnlineQuoteSource(_sourcename, m_profile->scriptPath(),
-                                       "\"([^,\"]*)\",.*", // symbol regexp
-                                       "[^,]*,[^,]*,\"([^\"]*)\"", // price regexp
-                                       "[^,]*,([^,]*),.*", // date regexp
-                                       "%y-%m-%d"); // date format
+    m_source = AlkOnlineQuoteSource(_sourcename,
+                                    m_profile->scriptPath(),
+                                    "\"([^,\"]*)\",.*", // symbol regexp
+                                    AlkOnlineQuoteSource::IdSelector::Symbol,
+                                    "[^,]*,[^,]*,\"([^\"]*)\"", // price regexp
+                                    "[^,]*,([^,]*),.*", // date regexp
+                                    "%y-%m-%d"); // date format
 
-    //emit status(QString("(Debug) symbol=%1 id=%2...").arg(_symbol,_id));
+    //Q_EMIT status(QString("(Debug) symbol=%1 id=%2...").arg(_symbol,_id));
     AlkFinanceQuoteProcess tmp;
     QString fQSource = m_profile->type() == AlkOnlineQuotesProfile::Type::Script ?
                 tmp.crypticName(_sourcename) : _sourcename.section(' ', 1);
@@ -519,7 +563,7 @@ bool AlkOnlineQuote::Private::launchFinanceQuote(const QString &_symbol, const Q
     args << "perl" << m_profile->scriptPath() << fQSource << m_symbol;
     m_filter.clearProgram();
     m_filter << args;
-    emit m_p->status(i18nc("Executing 'script' 'online source' 'investment symbol' ",
+    Q_EMIT m_p->status(i18nc("Executing 'script' 'online source' 'investment symbol' ",
                       "Executing %1 %2 %3...", args.join(" "), QString(), QString()));
 
     m_filter.setOutputChannelMode(KProcess::MergedChannels);
@@ -528,7 +572,7 @@ bool AlkOnlineQuote::Private::launchFinanceQuote(const QString &_symbol, const Q
     // This seems to work best if we just block until done.
     if (m_filter.waitForFinished()) {
     } else {
-        emit m_p->error(i18n("Unable to launch: %1", m_profile->scriptPath()));
+        Q_EMIT m_p->error(i18n("Unable to launch: %1", m_profile->scriptPath()));
         m_errors |= Errors::Script;
         result = slotParseQuote(QString());
     }
@@ -548,22 +592,22 @@ bool AlkOnlineQuote::Private::parsePrice(const QString &_pricestr)
     // set the last one to a period.
     QString pricestr(_pricestr);
     if (!pricestr.isEmpty()) {
-        int pos = pricestr.lastIndexOf(QRegExp("\\D"));
+        int pos = pricestr.lastIndexOf(Regex("\\D"));
         if (pos > 0) {
             pricestr[pos] = '.';
-            pos = pricestr.lastIndexOf(QRegExp("\\D"), pos - 1);
+            pos = pricestr.lastIndexOf(Regex("\\D"), pos - 1);
         }
         while (pos > 0) {
             pricestr.remove(pos, 1);
-            pos = pricestr.lastIndexOf(QRegExp("\\D"), pos);
+            pos = pricestr.lastIndexOf(Regex("\\D"), pos);
         }
 
         m_price = pricestr.toDouble();
         kDebug(Private::dbgArea()) << "Price" << pricestr;
-        emit m_p->status(i18n("Price found: '%1' (%2)", pricestr, m_price));
+        Q_EMIT m_p->status(i18n("Price found: '%1' (%2)", pricestr, m_price));
     } else {
         m_errors |= Errors::Price;
-        emit m_p->error(i18n("Unable to parse price for '%1'", m_symbol));
+        Q_EMIT m_p->error(i18n("Unable to parse price for '%1'", m_symbol));
         result = false;
     }
     return result;
@@ -572,34 +616,202 @@ bool AlkOnlineQuote::Private::parsePrice(const QString &_pricestr)
 bool AlkOnlineQuote::Private::parseDate(const QString &datestr)
 {
     if (!datestr.isEmpty()) {
-        emit m_p->status(i18n("Date found: '%1'", datestr));
+        Q_EMIT m_p->status(i18n("Date found: '%1'", datestr));
 
-        AlkDateFormat dateparse(m_source.dateformat());
+        AlkDateFormat dateparse(m_source.dateFormat());
         try {
             m_date = dateparse.convertString(datestr, false /*strict*/);
             kDebug(Private::dbgArea()) << "Date" << datestr;
-            emit m_p->status(i18n("Date format found: '%1' -> '%2'", datestr, m_date.toString()));
+            Q_EMIT m_p->status(i18n("Date format found: '%1' -> '%2'", datestr, m_date.toString()));
         } catch (const AlkException &e) {
             m_errors |= Errors::DateFormat;
-            emit m_p->error(i18n("Unable to parse date '%1' using format '%2': %3", datestr,
+            Q_EMIT m_p->error(i18n("Unable to parse date '%1' using format '%2': %3", datestr,
                                                                                dateparse.format(),
                                                                                e.what()));
             m_date = QDate::currentDate();
-            emit m_p->status(i18n("Using current date for '%1'", m_symbol));
+            Q_EMIT m_p->status(i18n("Using current date for '%1'", m_symbol));
         }
     } else {
-        if (m_source.date().isEmpty()) {
-            emit m_p->status(i18n("Parsing date is disabled for '%1'", m_symbol));
+        if (m_source.dateRegex().isEmpty()) {
+            Q_EMIT m_p->status(i18n("Parsing date is disabled for '%1'", m_symbol));
         } else {
             m_errors |= Errors::Date;
-            emit m_p->error(i18n("Unable to parse date for '%1'", m_symbol));
+            Q_EMIT m_p->error(i18n("Unable to parse date for '%1'", m_symbol));
         }
         m_date = QDate::currentDate();
-        emit m_p->status(i18n("Using current date for '%1'", m_symbol));
+        Q_EMIT m_p->status(i18n("Using current date for '%1'", m_symbol));
     }
     return true;
 }
 
+/**
+ * Parse quote data expected as stripped html
+ *
+ * @param quotedata quote data to parse
+ * @return true parsing successful
+ * @return false parsing unsuccessful
+ */
+bool AlkOnlineQuote::Private::parseQuoteStripHTML(const QString &_quotedata)
+{
+    QString quotedata = _quotedata;
+
+    //
+    // First, remove extraneous non-data elements
+    //
+
+    // HTML tags
+    quotedata.remove(Regex("<[^>]*>"));
+
+    // &...;'s
+    quotedata.replace(Regex("&\\w+;"), " ");
+
+    // Extra white space
+    quotedata = quotedata.simplified();
+    kDebug(Private::dbgArea()) << "stripped text" << quotedata;
+
+    return parseQuoteHTML(quotedata);
+}
+
+/**
+ * Parse quote data in html format
+ *
+ * @param quotedata quote data to parse
+ * @return true parsing successful
+ * @return false parsing unsuccessful
+ */
+bool AlkOnlineQuote::Private::parseQuoteHTML(const QString &quotedata)
+{
+    bool gotprice = false;
+    bool gotdate = false;
+    bool result = true;
+
+    QRegExp identifierRegExp(m_source.idRegex());
+    QRegExp dateRegExp(m_source.dateRegex());
+    QRegExp priceRegExp(m_source.priceRegex());
+
+    if (identifierRegExp.indexIn(quotedata) > -1) {
+        kDebug(Private::dbgArea()) << "Symbol" << identifierRegExp.cap(1);
+        Q_EMIT m_p->status(i18n("Symbol found: '%1'", identifierRegExp.cap(1)));
+    } else {
+        m_errors |= Errors::Symbol;
+        Q_EMIT m_p->error(i18n("Unable to parse symbol for %1", m_symbol));
+    }
+
+    if (priceRegExp.indexIn(quotedata) > -1) {
+        gotprice = true;
+        QString pricestr = priceRegExp.cap(1);
+        parsePrice(pricestr);
+    } else {
+        parsePrice(QString());
+    }
+
+    if (dateRegExp.indexIn(quotedata) > -1) {
+        QString datestr = dateRegExp.cap(1);
+        gotdate = parseDate(datestr);
+    } else {
+        gotdate = parseDate(QString());
+    }
+
+    if (gotprice && gotdate) {
+        Q_EMIT m_p->quote(m_id, m_symbol, m_date, m_price);
+    } else {
+        Q_EMIT m_p->failed(m_id, m_symbol);
+        result = false;
+    }
+    return result;
+}
+
+/**
+ * Parse quote data in csv format
+ *
+ * @param quotedata quote data to parse
+ * @return true parsing successful
+ * @return false parsing unsuccessful
+ */
+bool AlkOnlineQuote::Private::parseQuoteCSV(const QString &quotedata)
+{
+    QString dateColumn(m_source.dateRegex());
+    QString priceColumn(m_source.priceRegex());
+    QStringList lines = quotedata.split(QRegExp("\r?\n"));
+    QString header = lines.first();
+    QRegExp rx("([,;\t])");
+    QString columnSeparator;
+    if (rx.indexIn(header) != -1) {
+        columnSeparator = rx.cap(1);
+    }
+    if (columnSeparator.isEmpty()) {
+        m_errors |= Errors::Source;
+        Q_EMIT m_p->error(i18n("Unable to detect field delimiter in first line (header line) of quote data."));
+        Q_EMIT m_p->failed(m_id, m_symbol);
+        return false;
+    }
+    const QChar decimalSeparator = (columnSeparator.at(0) == ';') ? QLatin1Char(',') : QLatin1Char('.');
+
+    // detect column index
+    int dateCol = -1;
+    int priceCol = -1;
+    // check if column numbers are given
+    if (dateColumn.startsWith(QLatin1Char('#')) && priceColumn.startsWith(QLatin1Char('#'))) {
+        dateColumn.remove(0,1);
+        priceColumn.remove(0,1);
+        dateCol = dateColumn.toInt() - 1;
+        priceCol = priceColumn.toInt() - 1;
+    } else { // find columns
+        QStringList headerColumns = header.split(columnSeparator);
+        for (int i = 0; i < headerColumns.size(); i++) {
+            if (headerColumns[i].contains(dateColumn))
+                dateCol = i;
+            else if (headerColumns[i].contains(priceColumn))
+                priceCol = i;
+        }
+        lines.takeFirst();
+    }
+    if (dateCol == -1) {
+        m_errors |= Errors::Date;
+        Q_EMIT m_p->error(i18n("Unable to find date column '%1' in quote data", dateColumn));
+        Q_EMIT m_p->failed(m_id, m_symbol);
+        return false;
+    }
+    if (priceCol == -1) {
+        m_errors |= Errors::Price;
+        Q_EMIT m_p->error(i18n("Unable to find price column '%1' in quote data", priceColumn));
+        Q_EMIT m_p->failed(m_id, m_symbol);
+        return false;
+    }
+    AlkDatePriceMap prices;
+    for (const auto &line : lines) {
+        if (line.trimmed().isEmpty())
+            continue;
+        QStringList cols = line.split(columnSeparator);
+        QString dateValue = cols[dateCol].trimmed();
+        QString priceValue = cols[priceCol].trimmed();
+        if (dateValue.isEmpty() || priceValue.isEmpty())
+            continue;
+        // @todo: auto detect format
+        AlkDateFormat dateFormat(m_source.dateFormat());
+        QDate date = dateFormat.convertString(dateValue, false);
+        if (!date.isValid()) {
+            m_errors |= Errors::DateFormat;
+            Q_EMIT m_p->error(i18n("Unable to convert date '%1' with '%2' in quote data", dateValue, m_source.dateFormat()));
+            Q_EMIT m_p->failed(m_id, m_symbol);
+            return false;
+        }
+        if (!m_startDate.isNull() && date < m_startDate)
+            continue;
+        if (!m_endDate.isNull() && date > m_endDate)
+            continue;
+        prices[date] = AlkValue(priceValue, decimalSeparator);
+    }
+    if (prices.isEmpty()) {
+        m_errors |= Errors::Price;
+        Q_EMIT m_p->error(i18n("Unable to find date/price pairs in quote data"));
+        Q_EMIT m_p->failed(m_id, m_symbol);
+        return false;
+    }
+    Q_EMIT m_p->quotes(m_id, m_symbol, prices);
+    return true;
+}
+
 /**
  * Parse quote data according to currently selected web price quote source
  *
@@ -607,73 +819,36 @@ bool AlkOnlineQuote::Private::parseDate(const QString &datestr)
  * @return true parsing successful
  * @return false parsing unsuccessful
  */
-bool AlkOnlineQuote::Private::slotParseQuote(const QString &_quotedata)
+bool AlkOnlineQuote::Private::slotParseQuote(const QString &quotedata)
 {
-    QString quotedata = _quotedata;
     m_quoteData = quotedata;
-    bool gotprice = false;
-    bool gotdate = false;
-    bool result = true;
 
-    kDebug(Private::dbgArea()) << "quotedata" << _quotedata;
+    kDebug(Private::dbgArea()) << "quotedata" << quotedata;
 
-    if (!quotedata.isEmpty()) {
-        if (!m_source.skipStripping()) {
-            //
-            // First, remove extraneous non-data elements
-            //
-
-            // HTML tags
-            quotedata.remove(QRegExp("<[^>]*>"));
-
-            // &...;'s
-            quotedata.replace(QRegExp("&\\w+;"), " ");
-
-            // Extra white space
-            quotedata = quotedata.simplified();
-            kDebug(Private::dbgArea()) << "stripped text" << quotedata;
-        }
-
-        QRegExp symbolRegExp(m_source.sym());
-        QRegExp dateRegExp(m_source.date());
-        QRegExp priceRegExp(m_source.price());
-
-        if (symbolRegExp.indexIn(quotedata) > -1) {
-            kDebug(Private::dbgArea()) << "Symbol" << symbolRegExp.cap(1);
-            emit m_p->status(i18n("Symbol found: '%1'", symbolRegExp.cap(1)));
-        } else {
-            m_errors |= Errors::Symbol;
-            emit m_p->error(i18n("Unable to parse symbol for %1", m_symbol));
-        }
-
-        if (priceRegExp.indexIn(quotedata) > -1) {
-            gotprice = true;
-            QString pricestr = priceRegExp.cap(1);
-            parsePrice(pricestr);
-        } else {
-            parsePrice(QString());
-        }
-
-        if (dateRegExp.indexIn(quotedata) > -1) {
-            QString datestr = dateRegExp.cap(1);
-            gotdate = parseDate(datestr);
-        } else {
-            gotdate = parseDate(QString());
-        }
-
-        if (gotprice && gotdate) {
-            emit m_p->quote(m_id, m_symbol, m_date, m_price);
-        } else {
-            emit m_p->failed(m_id, m_symbol);
-            result = false;
-        }
-    } else {
+    if (quotedata.isEmpty()) {
         m_errors |= Errors::Data;
-        emit m_p->error(i18n("Unable to update price for %1 (empty quote data)", m_symbol));
-        emit m_p->failed(m_id, m_symbol);
-        result = false;
+        Q_EMIT m_p->error(i18n("Unable to update price for %1 (empty quote data)", m_symbol));
+        Q_EMIT m_p->failed(m_id, m_symbol);
+        return false;
     }
-    return result;
+    switch (m_source.dataFormat()) {
+    case AlkOnlineQuoteSource::StrippedHTML:
+        return parseQuoteStripHTML(quotedata);
+    case AlkOnlineQuoteSource::HTML:
+        return parseQuoteHTML(quotedata);
+    case AlkOnlineQuoteSource::CSV:
+        return parseQuoteCSV(quotedata);
+    default:
+        return false;
+    }
+}
+
+void AlkOnlineQuote::Private::slotLoadTimeout()
+{
+    Q_EMIT m_p->error(i18n("Timeout exceeded on fetching url for %1", m_symbol));
+    m_errors |= Errors::Timeout;
+    Q_EMIT m_p->failed(m_id, m_symbol);
+    m_eventLoop->exit(Errors::Timeout);
 }
 
 AlkOnlineQuote::AlkOnlineQuote(AlkOnlineQuotesProfile *profile, QObject *_parent)
@@ -722,6 +897,22 @@ void AlkOnlineQuote::setAcceptLanguage(const QString &language)
     d->m_acceptLanguage = language;
 }
 
+int AlkOnlineQuote::timeout() const
+{
+    return d->m_timeout;
+}
+
+void AlkOnlineQuote::setTimeout(int newTimeout)
+{
+    d->m_timeout = newTimeout;
+}
+
+void AlkOnlineQuote::setDateRange(const QDate &from, const QDate &to)
+{
+    d->m_startDate = from;
+    d->m_endDate = to;
+}
+
 bool AlkOnlineQuote::launch(const QString &_symbol, const QString &_id, const QString &_source)
 {
 #ifdef ENABLE_FINANCEQUOTE
