@@ -26,15 +26,17 @@
 #include <KConfigGroup>
 
 #if QT_VERSION >= QT_VERSION_CHECK(5,0,0)
-    #include <QStandardPaths>
-    #include <knscore/downloadmanager.h>
-    namespace KNS = KNSCore;
+#include <KNSCore/Engine>
+#include <KSharedConfig>
+#include <QStandardPaths>
+namespace KNS = KNSCore;
 #else
     #include <KGlobal>
     #include <KStandardDirs>
     #include <knewstuff3/downloadmanager.h>
     namespace KNS = KNS3;
 #endif
+#include <QRegExp>
 
 class AlkOnlineQuotesProfile::Private : public QObject
 {
@@ -46,8 +48,13 @@ public:
     QString m_GHNSFilePath;
     QString m_kconfigFile;
     AlkOnlineQuotesProfileManager *m_profileManager;
-    KNS::DownloadManager *m_manager;
-    KConfig *m_config;
+#if QT_VERSION < QT_VERSION_CHECK(5,0,0)
+    KNS::DownloadManager *m_manager = 0;
+    KConfig* m_config;
+#else
+    KNSCore::Engine *m_engine = 0;
+    KSharedConfigPtr m_config;
+#endif
     Type m_type;
     static QString m_financeQuoteScriptPath;
     static QStringList m_financeQuoteSources;
@@ -56,7 +63,8 @@ public:
     {
         if (m_financeQuoteScriptPath.isEmpty()) {
 #if QT_VERSION >= QT_VERSION_CHECK(5,0,0)
-            m_financeQuoteScriptPath = QStandardPaths::locate(QStandardPaths::AppDataLocation, QStringLiteral("misc/financequote.pl"));
+            m_financeQuoteScriptPath =
+                QStandardPaths::locate(QStandardPaths::GenericDataLocation, QStringLiteral("alkimia%1/misc/financequote.pl").arg(TARGET_SUFFIX));
 #else
             m_financeQuoteScriptPath = KGlobal::dirs()->findResource("appdata",
                                                                      QString("misc/financequote.pl"));
@@ -65,10 +73,9 @@ public:
         return !m_financeQuoteScriptPath.isEmpty();
     }
 
-    Private(AlkOnlineQuotesProfile *p)
+    explicit Private(AlkOnlineQuotesProfile *p)
         : m_p(p)
         , m_profileManager(0)
-        , m_manager(0)
         , m_config(0)
         , m_type(Type::Undefined)
     {
@@ -79,12 +86,15 @@ public:
 
     ~Private()
     {
+#if QT_VERSION < QT_VERSION_CHECK(5,0,0)
         delete m_manager;
         delete m_config;
+#endif
     }
 
     void checkUpdates()
     {
+#if QT_VERSION < QT_VERSION_CHECK(5,0,0)
         m_manager = new KNS::DownloadManager(m_p->hotNewStuffConfigFile(), this);
         // to know when checking for updates is done
         connect(m_manager, SIGNAL(searchResult(KNS3::Entry::List)), this,
@@ -94,14 +104,28 @@ public:
                 SLOT(entryStatusChanged(KNS3::Entry)));
         // start checking for updates
         m_manager->checkForUpdates();
+#else
+        m_engine = new KNSCore::Engine(this);
+        if (m_engine->init(m_p->hotNewStuffConfigFile())) {
+            connect(m_engine, &KNSCore::Engine::signalUpdateableEntriesLoaded, this, [this](const KNSCore::EntryInternal::List &updates) {
+                for (const KNSCore::EntryInternal &entry : updates) {
+                    qDebug() << "update available in profile" << m_p->name() << "for" << entry.name() << entry.version() << entry.uniqueId() << entry.category() << entry.providerId();
+                    Q_EMIT m_p->updateAvailable(m_p->name(), entry.name());
+                }
+            });
+            connect(m_engine, &KNSCore::Engine::signalProvidersLoaded, this, [this]() {
+                m_engine->checkForUpdates();
+            });
+        }
+#endif
     }
 
 public Q_SLOTS:
     void slotUpdatesFound(const KNS3::Entry::List &updates)
     {
-        foreach (const KNS3::Entry &entry, updates) {
+        for (const KNS3::Entry &entry : updates) {
             qDebug() << "update available in profile" << m_p->name() << "for" << entry.name() << entry.version() << entry.id() << entry.category() << entry.providerId();
-            emit m_p->updateAvailable(m_p->name(), entry.name());
+            Q_EMIT m_p->updateAvailable(m_p->name(), entry.name());
         }
     }
 
@@ -114,8 +138,7 @@ public Q_SLOTS:
     const QStringList quoteSourcesNative()
     {
         //KSharedConfigPtr kconfig = KGlobal::config();
-        KConfig config(m_kconfigFile);
-        KConfig *kconfig = &config;
+        auto kconfig = KSharedConfig::openConfig(m_kconfigFile, KConfig::SimpleConfig);
         QStringList groups = kconfig->groupList();
 
         QStringList::Iterator it;
@@ -154,12 +177,20 @@ public Q_SLOTS:
             // since this is a static function it can be called without constructing an object
             // so we need to make sure that m_financeQuoteScriptPath is properly initialized
             if (setupFinanceQuoteScriptPath()) {
-                AlkFinanceQuoteProcess getList;
-                getList.launch(m_financeQuoteScriptPath);
-                while (!getList.isFinished()) {
+                AlkFinanceQuoteProcess testList;
+                testList.testLaunch(m_financeQuoteScriptPath);
+                while (!testList.isFinished()) {
                     qApp->processEvents();
                 }
-                m_financeQuoteSources = getList.getSourceList();
+
+                if (testList.exitCode() == 0) {
+                    AlkFinanceQuoteProcess getList;
+                    getList.launch(m_financeQuoteScriptPath);
+                    while (!getList.isFinished()) {
+                        qApp->processEvents();
+                    }
+                    m_financeQuoteSources = getList.getSourceList();
+                }
             }
         }
         return m_financeQuoteSources;
@@ -188,7 +219,7 @@ public Q_SLOTS:
         const QString filename = QString("%1/*.txt").arg(m_GHNSFilePath);
         const QStringList resources = KStandardDirs().findAllResources("data", filename);
 #endif
-        foreach (const QString &file, resources) {
+        for (const QString &file : resources) {
             QFileInfo f(file);
             QString file2 = f.completeBaseName();
             AlkOnlineQuoteSource source(file2, m_p);
@@ -217,25 +248,11 @@ public Q_SLOTS:
         case AlkOnlineQuotesProfile::Type::None:
         case AlkOnlineQuotesProfile::Type::Alkimia4:
         case AlkOnlineQuotesProfile::Type::Alkimia5: {
-            AlkOnlineQuoteSource source("Alkimia Currency",
-                                        "https://fx-rate.net/%1/%2",
-                                        QString(), // symbolregexp
-                                        "1[ a-zA-Z]+=</span><br */?> *(\\d+[\\.\\d]*)",
-                                        "updated\\s\\d+:\\d+:\\d+\\(\\w+\\)\\s+(\\d{1,2}/\\d{2}/\\d{4})",
-                                        "%d/%m/%y",
-                                        true // skip HTML stripping
-                                        );
+            AlkOnlineQuoteSource source(AlkOnlineQuoteSource::defaultCurrencyQuoteSource("Alkimia Currency"));
             source.setProfile(m_p);
             result[source.name()] = source;
 #if defined(BUILD_WITH_WEBKIT) || defined(BUILD_WITH_WEBENGINE)
-            AlkOnlineQuoteSource source2("Alkimia Currency.webkit",
-                                        "https://fx-rate.net/%1/%2",
-                                        QString(), // symbolregexp
-                                        "1[ a-zA-Z]+=</span><br */?> *(\\d+[\\.\\d]*)",
-                                        "", // no date available
-                                        "",
-                                        true // skip HTML stripping
-                                        );
+            AlkOnlineQuoteSource source2(AlkOnlineQuoteSource::defaultCurrencyQuoteSource("Alkimia Currency.webkit"));
             source2.setProfile(m_p);
             result[source2.name()] = source2;
 #endif
@@ -330,9 +347,17 @@ AlkOnlineQuotesProfile::AlkOnlineQuotesProfile(const QString &name, Type type,
     else if (type == Type::Alkimia5 || type == Type::Alkimia4)
         d->m_kconfigFile = QString("%1/alkimiarc").arg(d->configPath());
     else
-        d->m_kconfigFile = "";
-    if (!d->m_kconfigFile.isEmpty())
+        d->m_kconfigFile.clear();
+
+    if (!d->m_kconfigFile.isEmpty()) {
+#if QT_VERSION >= QT_VERSION_CHECK(5, 0, 0)
+        d->m_config = KSharedConfig::openConfig(d->m_kconfigFile, KConfig::SimpleConfig);
+        qDebug() << (void*)d->m_config.constData();
+#else
         d->m_config = new KConfig(d->m_kconfigFile);
+#endif
+    }
+
     if (!d->m_GHNSFile.isEmpty()) {
         KConfig ghnsFile(hotNewStuffConfigFile());
         KConfigGroup group = ghnsFile.group("KNewStuff3");
@@ -373,7 +398,7 @@ QString AlkOnlineQuotesProfile::hotNewStuffConfigFile() const
 
 QString AlkOnlineQuotesProfile::hotNewStuffReadFilePath(const QString &fileName) const
 {
-    foreach(const QString &path, hotNewStuffReadPath()) {
+    for (const QString &path : hotNewStuffReadPath()) {
         QFileInfo f(path + fileName);
         if (f.exists())
             return f.absoluteFilePath();
@@ -408,16 +433,51 @@ QString AlkOnlineQuotesProfile::kConfigFile() const
     return d->m_kconfigFile;
 }
 
+#if QT_VERSION >= QT_VERSION_CHECK(5, 0, 0)
+KSharedConfigPtr AlkOnlineQuotesProfile::kConfig() const
+#else
 KConfig *AlkOnlineQuotesProfile::kConfig() const
+#endif
 {
     return d->m_config;
 }
 
+#if QT_VERSION >= QT_VERSION_CHECK(5, 0, 0)
+void AlkOnlineQuotesProfile::setKConfig(KSharedConfigPtr kconfig)
+{
+    d->m_config = kconfig;
+}
+#endif
+
 AlkOnlineQuotesProfile::Type AlkOnlineQuotesProfile::type()
 {
     return d->m_type;
 }
 
+bool AlkOnlineQuotesProfile::typeIsSupported() const
+{
+#ifndef ENABLE_FINANCEQUOTE
+    if (d->m_type == Type::Script) {
+        return false;
+    }
+#endif
+    return true;
+}
+
+bool AlkOnlineQuotesProfile::typeIsOperational() const
+{
+    if (d->m_type == Type::Script) {
+#ifdef ENABLE_FINANCEQUOTE
+        if (d->quoteSourcesFinanceQuote().isEmpty()) {
+            return false;
+        }
+#else
+        return false;
+#endif
+    }
+    return true;
+}
+
 bool AlkOnlineQuotesProfile::hasGHNSSupport()
 {
     return !d->m_GHNSFile.isEmpty();
