@@ -22,7 +22,6 @@ class ALK_EXPORT AlkOnlineQuotesProfileManager : public QObject
 {
     Q_OBJECT
 public:
-    AlkOnlineQuotesProfileManager();
     ~AlkOnlineQuotesProfileManager();
 
     void addProfile(AlkOnlineQuotesProfile *profile);
@@ -33,8 +32,18 @@ public:
     void setWebPageEnabled(bool enable);
     bool webPageEnabled();
 
+    /**
+     * Return instance of this class
+     * @return instance
+     */
     static AlkOnlineQuotesProfileManager &instance();
 
+    /**
+     * Return pointer to the singleton instance of this class
+     * @return pointer to instance
+     */
+    static AlkOnlineQuotesProfileManager *instancePointer();
+
 Q_SIGNALS:
     /**
      * emit quote source for which an update is available
@@ -42,6 +51,7 @@ Q_SIGNALS:
     void updateAvailable(const QString &profile, const QString &name);
 
 private:
+    AlkOnlineQuotesProfileManager();
     class Private;
     Private *const d;
 };
