@@ -80,6 +80,7 @@ MainWindow::MainWindow(QWidget *parent)
 #endif
     d->ui.setupUi(this);
     d->quotesWidget = new AlkOnlineQuotesWidget(true, true);
+    d->quotesWidget->setGHNSSourceEditable(true);
 
     QDockWidget *profilesWidget = new QDockWidget(i18n("Profiles"), this);
     profilesWidget->setObjectName("profilesDockWidget");
@@ -106,8 +107,8 @@ MainWindow::MainWindow(QWidget *parent)
     quoteDetailsWidget->setWidget(d->quotesWidget->quoteDetailsWidget());
     addDockWidget(Qt::RightDockWidgetArea, quoteDetailsWidget);
 
-    connect(&AlkOnlineQuotesProfileManager::instance(), SIGNAL(updateAvailable(const QString &, const QString &)),
-            this, SLOT(slotUpdateAvailable(const QString &, const QString &)));
+    connect(&AlkOnlineQuotesProfileManager::instance(), SIGNAL(updateAvailable(QString,QString)),
+            this, SLOT(slotUpdateAvailable(QString,QString)));
 
     manager.setWebPageEnabled(true);
     QDockWidget *browserWidget = new QDockWidget(i18n("Browser"), this);
@@ -129,7 +130,7 @@ MainWindow::MainWindow(QWidget *parent)
             QLocale::AnyCountry);
 
     QStringList languages;
-    foreach(const QLocale &locale, allLocales) {
+    for (const QLocale &locale : allLocales) {
         languages.append(locale.uiLanguages());
     }
     languages.sort();
